//This file implements the List class

//Suggested Further Exercises: 

//1. Add a tail member to the private
//data members of the List class
//If you do this what List method becomes trivial
//to implement ?

//2. Implement a List method Reverse
//that reverses a list (header given in List.h)


#include <iostream>
#include "List.h"

using namespace std;

//List Constructor
//Initializes the head of the list

template<class Type>
List<Type> :: List() {
  head = NULL;
}

//List Copy constructor
//Performs a "deep" copy of the List l given
//as argument:
//For each node in the l list creates a new node, copies its value
//and links it into a new list. The head member of the current List 
//object is assigned the newly constructed list.

template<class Type>
List<Type> :: List (const List<Type>& l){
 
  if(!l.head) {
    head = NULL;
  }
  else
    {
      head = new ListNode<Type> (l.head->GetVal(), NULL);
      if (head == NULL) exit (0);

      ListNode<Type> *last = head;
      for (ListNode<Type> *crt = l.head->GetNext(); crt != NULL; crt = crt->GetNext()){
		ListNode<Type> *tmp;
		tmp = new ListNode<Type>(crt->GetVal(), NULL);
		last->SetNext(tmp);
		last = tmp;
      }
    } 
}

//List Destructor:
//Deallocates space that was allocated to each node
//in the current list

template<class Type>
List<Type>::~List() {
  ListNode<Type> *tmp;
  for (ListNode<Type> *crt = head; crt != NULL; crt = tmp){
    tmp = crt->GetNext();
    delete crt;
  }
}

//List Public Method Tail
//Returns a pointer to the list's last node
//or NULL if the list is empty

template<class Type>
ListNode<Type> * List<Type> :: Tail() {

  if (head == NULL)
	return NULL;

  ListNode<Type> *prev, *crt;
  for (prev = NULL, crt = head; crt != NULL; crt = crt->GetNext()){
    prev = crt;
  }
  return prev;
}



//List Public Method AddNode
//Adds a node to the current List at the beginning
//Creates a new List node 
//Note that the ListNode constructor is invoked.
//Updates head to point to the newly created node
   
template<class Type>
void List<Type> :: AddNode (const Type &val){
      ListNode<Type> *tmp = new ListNode<Type>(val, head);
      if (tmp == NULL) exit (0);
      head = tmp;
}


//List Public Method AppendNode
//Adds a node to the current List at the end
//Needs to get to the end of the list first
//creates a new node and updates the last
//node in the list to point to the newly created node.

template<class Type>   
void List<Type> :: AppendNode (const Type &val){
      ListNode<Type> *tmp = new ListNode<Type>(val, NULL);
      if (tmp == NULL) exit (0);
	  ListNode<Type> *last = Tail();
	  last->SetNext(tmp);
}



//List Public Method Display
//Displays the current List

template<class Type>
void List<Type> :: Display () const {
  cout << "List content:";
  for (ListNode<Type> *crt= head; crt != NULL; crt = crt->GetNext())
    cout << " " << crt->GetVal();
  cout << endl;
}
      
template class List<int>;
template class List<char *>;
